from noseOfYeti.plugins.support.test_chooser import TestChooser
from should_dsl import should

# Silencing code checker about should_dsl matchers
be = None
equal_to = None

class Test_TestChooser(object):

    def setup(self):
        self.test_chooser = TestChooser()

    def test_it_resets_done_when_told_about_new_module(self):
        self.test_chooser.done |should| equal_to({})
        self.test_chooser.done['a'] = 3
        self.test_chooser.done |should| equal_to({'a':3})
        self.test_chooser.new_module()
        self.test_chooser.done |should| equal_to({})

    def test_already_visited_puts_kls_name_key_in_done_or_returns_True(self):
        self.test_chooser.done |should| equal_to({})
        self.test_chooser.already_visited('a', 'b') |should| be(False)
        self.test_chooser.done |should| equal_to({'a.b' : True})
        self.test_chooser.already_visited('a', 'b') |should| be(True)

        self.test_chooser.already_visited('c', 'd') |should| be(False)
        self.test_chooser.done |should| equal_to({'a.b' : True, 'c.d' : True})
        self.test_chooser.already_visited('c', 'd') |should| be(True)

class Test_TestChooser_Consider(object):
    __childof__ = Test_TestChooser
    __testname__ = ".consider() method"

    def setup(self):
        self.test_chooser = TestChooser()
        class TestKlsForTest(object):
            def ignore__test(self): pass

            def test_with__test__set(self): pass
            test_with__test__set.__test__ = False

            def test_actual(self): pass

        class TestIgnoredKls(object):
            def test_things(self): pass

        class TestKlsWithInherited(TestKlsForTest):
            def test_on_subclass(self): pass

        self.TestKlsForTest = TestKlsForTest
        self.TestIgnoredKls = TestIgnoredKls
        self.TestKlsWithInherited = TestKlsWithInherited

    def test_it_ignores_if_method_starts_with_ignore(self):
        self.test_chooser.consider(self.TestKlsForTest().ignore__test) |should| be(False)

    def test_it_ignores_if_method_has__test__set_to_false(self):
        self.test_chooser.consider(self.TestKlsForTest().test_with__test__set) |should| be(False)

    def test_it_ignores_if_method_has_kls_in_ignoreKls(self):
        self.TestIgnoredKls.is_noy_spec = True
        self.test_chooser.consider(self.TestIgnoredKls().test_things) |should| be(True)
        self.test_chooser.consider(self.TestIgnoredKls().test_things, ignore_kls=['TestIgnoredKls']) |should| be(False)

    def test_it_returns_None_if_kls_does_not_have_is_noy_test_set(self):
        self.test_chooser.consider(self.TestKlsForTest().test_actual) |should| be(None)

    def test_it_ignores_inherited_tests_if_is_noy_test_is_set_on_kls(self):
        self.test_chooser.consider(self.TestKlsWithInherited().test_actual) |should| be(None)
        self.TestKlsWithInherited.is_noy_spec = True
        self.test_chooser.consider(self.TestKlsWithInherited().test_actual) |should| be(False)
        self.test_chooser.consider(self.TestKlsWithInherited().test_on_subclass) |should| be(True)

    def test_it_ignores_functions_already_visited(self):
        self.TestKlsWithInherited.is_noy_spec = True
        self.test_chooser.consider(self.TestKlsWithInherited().test_on_subclass) |should| be(True)
        self.test_chooser.consider(self.TestKlsWithInherited().test_on_subclass) |should| be(False)




"""
To be used as a pylint plugin to register the spec encoding
"""
from pylint.interfaces import IASTNGChecker
from pylint.checkers import BaseChecker

from noseOfYeti.tokeniser.spec_codec import register_from_options
from noseOfYeti.tokeniser.config import Default
from support.spec_options import spec_options

import os

class Empty(object): pass

def normalise_options(template):
    env = os.environ
    parser_options = ['default', 'help', 'type']
    for option, attributes in template.items():
        opts = dict((k, v) for k, v in attributes.items() if k in parser_options)
        opts['default'] = opts['default'](env)
        yield option, opts

def make_extractor(non_default):
    """
        Return us a function to extract options
        Anything not in non_default is wrapped in a "Default" object
    """
    def extract_options(template, options):
        for option, val in normalise_options(template):
            name = option.replace('-', '_')

            value = getattr(options, name)
            if option not in non_default:
                value = Default(value)

            yield name, value
    return extract_options

class SpecRegister(BaseChecker):

    name = 'NoyRegister'
    __implements__ = IASTNGChecker

    msgs = {'W6969' : ('', '')}
    options = list(normalise_options(spec_options))
    priority = -1

    def __init__(self, *args, **kwargs):
        self.specified = []
        super(SpecRegister, self).__init__(*args, **kwargs)

    def set_option(self, name, val, action=Empty, opts=Empty):
        """Determine which options were specified outside of the defaults"""
        if action is Empty and opts is Empty:
            self.specified.append(name)
            super(SpecRegister, self).set_option(name, val)
        else:
            super(SpecRegister, self).set_option(name, val, action, opts)

    def open(self):
        options = self.config
        register_from_options(options, spec_options, extractor=make_extractor(self.specified))

def register(linter):
    linter.register_checker(SpecRegister(linter))




import json
import os

class MissingConfigFile(Exception): pass

########################
###   DEFAULT
########################

class Default(object):
	"""
		Wrapper for default values
		So we can tell the difference between default and no value
	"""
	def __init__(self, val):
		self.val = val

	def __str__(self):
		return str(self.val)

	def __unicode__(self):
		return unicode(self.val)

	def append(self, val):
		"""Make sure argparse doesn't fail"""
		self.val.append(val)

########################
###   CONFIGUTIL
########################

class ConfigUtil(object):
	"""Config functionality"""
	def __init__(self, template=None):
		self.values = {}
		self.template = template

	def normalise_key(self, key):
		"""Make sure key is a valid python attribute"""
		key = key.replace('-', '_')
		if key.startswith("noy_"):
			key = key[4:]
		return key

	def find_value(self, key):
		"""Find a value and return it"""
		values = self.values
		if key not in values:
			raise AttributeError("Config has no value for {}".format(key))

		val = values[key]
		if isinstance(val, Default):
			return val.val
		else:
			return val

	def use_options(self, options, extractor=None):
		"""
			If extractor isn't specified, then just update self.values with options.

			Otherwise update values with whatever the result of calling extractor with
			our template and these options returns

			Also make sure all keys are transformed into valid python attribute names
		"""
		# Extract if necessary
		if not extractor:
			extracted = options
		else:
			extracted = extractor(self.template, options)

		# Get values as [(key, val), ...]
		if isinstance(extracted, dict):
			extracted = extracted.items()

		# Add our values if there are any
		# Normalising the keys as we go along
		if extracted is not None:
			for key, val in extracted:
				self.values[self.normalise_key(key)] = val

	def find_config_file(self):
		"""
			Find where our config file is if there is any

			If the value for the config file is a default and it doesn't exist
			then it is silently ignored.

			If however, the value isn't a default and it doesn't exist, an error is raised
		"""
		filename = self.values.get('config_file', Default('noy.json'))

		ignore_missing = False
		if isinstance(filename, Default):
			filename = filename.val
			ignore_missing = True

		filename = os.path.abspath(filename)
		if os.path.exists(filename):
			return filename
		elif not ignore_missing:
			raise MissingConfigFile("Config file doesn't exist at {}".format(filename))

	def apply_config_file(self, filename):
		"""
			Add options from config file to self.values
			Leave alone existing values that are not an instance of Default
		"""
		def extractor(template, options):
			"""Ignore things that are existing non default values"""
			for name, val in options:
				normalised = self.normalise_key(name)
				if normalised in self.values and not isinstance(self.values[normalised], Default):
					continue
				else:
					yield name, val

		items = json.load(open(filename)).items()
		self.use_options(items, extractor)

	def use_config_file(self):
		"""Find and apply the config file"""
		self.config_file = self.find_config_file()
		if self.config_file:
			self.apply_config_file(self.config_file)

########################
###   CONFIG
########################

class Config(object):
	"""
		A wrapper around ConfigUtil.

		This provides a setup function that knows how to initialise from some options
		And how to find a config file and apply those values
		(config file values are overwritten by the options passed into setup)

		It then provides a __getattr__ to access the values on the ConfigUtil.
	"""
	def __init__(self, template=None):
		self._util = ConfigUtil(template)

	def setup(self, options=None, extractor=None):
		"""
			Put options onto the config and put anything from a config file onto the config.

			If extractor is specified, it is used to extract values from the options dictionary
		"""
		# Get our programmatic options
		self._util.use_options(options, extractor)

		# Overwrite non defaults in self.values with values from config
		self._util.use_config_file()

	def __getattr__(self, key):
		"""Get us attributes from the class or from self.values"""
		if key.startswith("_") or key in self.__dict__:
			return object.__getattribute__(self, key)

		return self._util.find_value(key)




'''
Tests to make sure registering the spec codec actually works
'''
from should_dsl import should
import subprocess
import sys
import os

from helpers import a_temp_file, a_temp_dir

# Silencing code checker about should_dsl matchers
be = None
equal_to = None

init_codez = "from noseOfYeti.tokeniser.spec_codec import register_from_options; register_from_options()"

example_specd_tests = """
# coding: spec

describe "blah":
	it "should totally work":
		1 + 1 |should| equal_to(2)

print ' '.join([key for key in TestBlah.__dict__.keys() if hasattr(getattr(TestBlah, key), 'im_func')])
""".strip()

class Test_RegisteringCodec(object):
	def test_not_registering_codec_leads_to_error(self):
		with a_temp_file(example_specd_tests) as filename:
			process = subprocess.Popen([sys.executable, filename], stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
			process.wait()
			process.returncode |should| be(1)

			expected_output = 'File "{}", line 1\nSyntaxError: encoding problem: with BOM'.format(filename)
			process.stdout.read().strip() |should| equal_to(expected_output)

	def test_registering_codec_doesnt_lead_to_error(self):
		with a_temp_dir() as tempdir:
			with open(os.path.join(tempdir, "codez.py"), 'w') as fle:
				fle.write(example_specd_tests)

			with open(os.path.join(tempdir, "spec_setup.py"), 'w') as fle:
				fle.write(init_codez)

			filename = os.path.join(tempdir, "script.py")
			with open(filename, 'w') as fle:
				fle.write("import spec_setup; import codez")

			process = subprocess.Popen([sys.executable, filename], cwd=tempdir, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
			process.wait()
			if process.returncode != 0:
				print process.stdout.read()
			process.returncode |should| be(0)

			expected_output = 'test_should_totally_work'
			process.stdout.read().strip() |should| equal_to(expected_output)




from tokenize import untokenize
from encodings import utf_8
import cStringIO
import encodings
import codecs
import re

from imports import determine_imports
from tokeniser import Tokeniser
from config import Config

regexes = {
      'whitespace' : re.compile('\s*')
    , 'only_whitespace' : re.compile('^\s*$')
    , 'encoding_matcher' : re.compile("#\s*coding\s*:\s*spec")
    , 'leading_whitespace' : re.compile('^(\s*)[^\s]')
    }

class TokeniserCodec(object):
    """Class to register the spec codec"""
    def __init__(self, tokeniser):
        self.tokeniser = tokeniser

    def register(self):
        """Register spec codec"""
        class StreamReader(utf_8.StreamReader):
            """Used by cPython to deal with a spec file"""
            def __init__(sr, stream, *args, **kwargs):
                codecs.StreamReader.__init__(sr, stream, *args, **kwargs)
                data = self.dealwith(sr.stream.readline)
                sr.stream = cStringIO.StringIO(data)

        def decode(text, *args):
            """Used by pypy and pylint to deal with a spec file"""
            buffered = cStringIO.StringIO(text)

            # Determine if we need to have imports for this string
            # It may be a fragment of the file
            has_spec = regexes['encoding_matcher'].search(buffered.readline())
            no_imports = not has_spec
            buffered.seek(0)

            # Translate the text
            utf8 = encodings.search_function('utf8') # Assume utf8 encoding
            reader = utf8.streamreader(buffered)
            data = self.dealwith(reader.readline, no_imports=no_imports)

            # If nothing was changed, then we want to use the original file/line
            # Also have to replace indentation of original line with indentation of new line
            # To take into account nested describes
            if text and not regexes['only_whitespace'].match(text):
                if regexes['whitespace'].sub('', text) == regexes['whitespace'].sub('', data):
                    bad_indentation = regexes['leading_whitespace'].search(text).groups()[0]
                    good_indentation = regexes['leading_whitespace'].search(data).groups()[0]
                    data = '%s%s' % (good_indentation, text[len(bad_indentation):])

            # If text is empty and data isn't, then we should return text
            if len(text) == 0 and len(data) == 1:
                return unicode(text), 0

            # Return translated version and it's length
            return unicode(data), len(data)

        def search_function(s):
            """Determine if a file is of spec encoding and return special CodecInfo if it is"""
            if s != 'spec': return None
            utf8 = encodings.search_function('utf8') # Assume utf8 encoding
            return codecs.CodecInfo(
                  name='spec'
                , encode=utf8.encode
                , decode=decode
                , streamreader=StreamReader
                , streamwriter=utf8.streamwriter
                , incrementalencoder=utf8.incrementalencoder
                , incrementaldecoder=utf8.incrementaldecoder
                )

        # Do the register
        codecs.register(search_function)

    def dealwith(self, readline, **kwargs):
        """
            Replace the contents of spec file with the translated version
            readline should be a callable object
            , which provides the same interface as the readline() method of built-in file objects
        """
        data = []
        try:
            # We pass in the data variable as an argument so that we
            # get partial output even in the case of an exception.
            self.tokeniser.translate(readline, data, **kwargs)
        except Exception as e:
            # Comment out partial output so that it doesn't result in
            # a syntax error when received by the interpreter.
            lines = []
            for line in untokenize(data).split('\n'):
                lines.append("# %s" % line)

            # Create exception to put into code to announce error
            exception = 'raise Exception("""--- internal spec codec error --- %s""")' % e

            # Need to make sure the exception doesn't add a new line and put out line numberes
            if len(lines) == 1:
                data = "%s%s" % (exception, lines[0])
            else:
                lines.append(exception)
                first_line = lines.pop()
                lines[0] = "%s %s" % (first_line, lines[0])
                data = '\n'.join(lines)
        else:
            # At this point, data is a list of tokens
            data = untokenize(data)

        return data

    def output_for_debugging(self, stream, data):
        """It will write the translated version of the file"""
        with open('%s.spec.out' % stream.name, 'w') as f: f.write(str(data))

########################
###   CODEC REGISTER
########################

def register_from_options(options=None, template=None, extractor=None):
    """Register the spec codec using the provided options"""
    if template is None:
        from noseOfYeti.plugins.support.spec_options import spec_options as template

    if extractor is None:
        from noseOfYeti.plugins.support.spec_options import extract_options_dict as extractor

    config = Config(template)
    config.setup(options, extractor)

    imports = determine_imports(
          extra_imports = ';'.join([d for d in config.extra_import if d])
        , with_should_dsl = config.with_should_dsl
        , with_default_imports = config.with_default_imports
        )

    tok = Tokeniser(
          default_kls = config.default_kls
        , import_tokens = imports
        , wrapped_setup = config.wrapped_setup
        , with_describe_attrs = not config.no_describe_attrs
        )

    TokeniserCodec(tok).register()




from noseOfYeti.tokeniser.imports import determine_imports
from should_dsl import should
from tokenize import NAME, OP

# Silencing code checker about should_dsl matchers
equal_to = None

class Test_DetermineImports(object):
    def test_extra_imports_are_added(self):
        extra_imports = "import thing; import stuff"
        determine_imports(
            extra_imports=extra_imports, with_default_imports=False, with_should_dsl=False
        ) |should| equal_to([
            (NAME, 'import'), (NAME, 'thing'), (OP, ';'), (NAME, 'import'), (NAME, 'stuff')
        ])

    def test_extra_imports_added_before_defaults(self):
        extra_imports = "import thing; import stuff"
        determine_imports(
            extra_imports=extra_imports, with_default_imports=True, with_should_dsl=False
        ) |should| equal_to([
              (NAME, 'import'), (NAME, 'thing'), (OP, ';'), (NAME, 'import'), (NAME, 'stuff')
            , (OP, ';') # Extra semicolon inserted
            , (NAME, 'import'), (NAME, 'nose'), (OP, ';')
            , (NAME, 'from'), (NAME, 'nose'), (OP, '.'), (NAME, 'tools'), (NAME, 'import'), (OP, '*'), (OP, ';')
            , (NAME, 'from')
                , (NAME, 'noseOfYeti'), (OP, '.'), (NAME, 'tokeniser'), (OP, '.'), (NAME, "support")
                , (NAME, 'import'), (OP, '*')
        ])

    def test_extra_imports_not_added_if_no_defaults(self):
        determine_imports(with_default_imports=True, with_should_dsl=False) |should| equal_to([
              (NAME, 'import'), (NAME, 'nose'), (OP, ';')
            , (NAME, 'from'), (NAME, 'nose'), (OP, '.'), (NAME, 'tools'), (NAME, 'import'), (OP, '*'), (OP, ';')
            , (NAME, 'from')
                , (NAME, 'noseOfYeti'), (OP, '.'), (NAME, 'tokeniser'), (OP, '.'), (NAME, "support")
                , (NAME, 'import'), (OP, '*')
        ])

    def test_should_dsl_imports_added_if_specified(self):
        determine_imports(with_default_imports=True, with_should_dsl=True) |should| equal_to([
              (NAME, 'import'), (NAME, 'nose'), (OP, ';')
            , (NAME, 'from'), (NAME, 'nose'), (OP, '.'), (NAME, 'tools'), (NAME, 'import'), (OP, '*'), (OP, ';')
            , (NAME, 'from'), (NAME, 'should_dsl'), (NAME, 'import'), (OP, '*'), (OP, ';')
            , (NAME, 'from')
                , (NAME, 'noseOfYeti'), (OP, '.'), (NAME, 'tokeniser'), (OP, '.'), (NAME, "support")
                , (NAME, 'import'), (OP, '*')
        ])

    def test_it_returns_nothing_if_no_imports(self):
        determine_imports(with_default_imports=False, with_should_dsl=False) |should| equal_to([])




from noseOfYeti.tokeniser import Tokeniser
from should_dsl import should

# Silencing code checker about should_dsl matchers
equal_to = None
result_in = None

class Test_Tokeniser_Nesting(object):
    def setUp(self):
        self.toka = Tokeniser(with_describe_attrs=False)
        self.tokb = Tokeniser(with_describe_attrs=False, default_kls = 'other')

        ###   SMALL EXAMPLE (WITHOUT PASS)

        self.small_example = [
        '''
        describe "This":
            describe "That":

                describe "Meh":pass
            context "Blah":pass
        describe "Another":pass '''
        ,
        '''
        class TestThis (%(o)s ):pass
        class TestThis_That (TestThis ):pass
        class TestThis_That_Meh (TestThis_That ):pass
        class TestThis_Blah (TestThis ):pass
        class TestAnother (%(o)s ):pass
        '''
        ]

        ###   SMALL EXAMPLE (WITH PATH FOR BW COMPAT)

        self.small_example_with_pass = [
        '''
        context "This":pass
            describe "That":pass
                describe "Meh":pass
            describe "Blah":pass
        describe "Another":pass '''
        ,
        '''
        class TestThis (%(o)s ):pass
        class TestThis_That (TestThis ):pass
        class TestThis_That_Meh (TestThis_That ):pass
        class TestThis_Blah (TestThis ):pass
        class TestAnother (%(o)s ):pass
        '''
        ]

        ###   BIG EXAMPLE

        self.big_example = [
        '''
        describe "This":
            it 'should':
                if x:
                    pass
                else:
                    x += 9
            describe "That":
                describe "Meh":
                    it 'should':
                        if y:
                            pass
                        else:
                            pass
            describe "Blah":pass
        describe "Another":
            it 'should':
                if z:
                    if u:
                        print "hello \
                            there"
                    else:
                        print "no"
                else:
                    pass
        '''
        ,
        '''
        class TestThis (%(o)s ):
            def test_should (self ):
                if x :
                    pass
                else :
                    x +=9
        class TestThis_That (TestThis ):pass
        class TestThis_That_Meh (TestThis_That ):
            def test_should (self ):
                if y :
                    pass
                else :
                    pass
        class TestThis_Blah (TestThis ):pass
        class TestAnother (%(o)s ):
            def test_should (self ):
                if z :
                    if u :
                        print "hello \
                            there"
                    else :
                        print "no"
                else :
                    pass
        '''
        ]

    ###   TESTS

    def test_works_with_space(self):
        test, desired = self.small_example
        (self.toka, test) |should| result_in(desired % {'o' : 'object'})
        (self.tokb, test) |should| result_in(desired % {'o' : 'other'})

    def test_works_with_tabs(self):
        test, desired = [d.replace('    ', '\t') for d in self.small_example]
        (self.toka, test) |should| result_in(desired % {'o' : 'object'})
        (self.tokb, test) |should| result_in(desired % {'o' : 'other'})

    def test_works_with_space_and_inline_pass(self):
        test, desired = self.small_example_with_pass
        (self.toka, test) |should| result_in(desired % {'o' : 'object'})
        (self.tokb, test) |should| result_in(desired % {'o' : 'other'})

    def test_works_with_tabs_and_inline_pass(self):
        test, desired = [d.replace('    ', '\t') for d in self.small_example_with_pass]
        (self.toka, test) |should| result_in(desired % {'o' : 'object'})
        (self.tokb, test) |should| result_in(desired % {'o' : 'other'})

    def test_keeps_good_indentation_in_body_with_spaces(self):
        test, desired = self.big_example
        (self.toka, test) |should| result_in(desired % {'o' : 'object'})
        (self.tokb, test) |should| result_in(desired % {'o' : 'other'})

    def test_keeps_good_indentation_in_body_with_tabs(self):
        test, desired = [d.replace('    ', '\t') for d in self.big_example]
        (self.toka, test) |should| result_in(desired % {'o' : 'object'})
        (self.tokb, test) |should| result_in(desired % {'o' : 'other'})

    def test_names_nested_describes_with_part_of_parents_name(self):
        test = 'describe "a":\n\tdescribe "b":'
        desired = 'class TestA (object ):pass\nclass TestA_B (TestA ):'
        (self.toka, test) |should| result_in(desired)




"""
A helper module to access the superclass'
setUp() and tearDown() methods of generated
test classes.

NB: We pull this into it's own module to:
1) not repeat these methods in all generated modules
2) avoid errors when inspecting the stack (e.g. in flexmock)
"""

from functools import wraps

def noy_sup_setUp(sup):
    if hasattr(sup, "setup"):
        return sup.setup()

    if hasattr(sup, "setUp"):
        return sup.setUp()

def noy_sup_tearDown(sup):
    if hasattr(sup, "teardown"):
        return sup.teardown()

    if hasattr(sup, "tearDown"):
        return sup.tearDown()

def noy_wrap_setUp(kls, func):
    @wraps(func)
    def wrapped(self, *args, **kwargs):
        sup = super(kls, self)
        noy_sup_setUp(sup)
        return func(self, *args, **kwargs)
    return wrapped

def noy_wrap_tearDown(kls, func):
    @wraps(func)
    def wrapped(self, *args, **kwargs):
        sup = super(kls, self)
        noy_sup_tearDown(sup)
        return func(self, *args, **kwargs)
    return wrapped




from tokenize import NAME, OP, INDENT, NEWLINE, STRING
from tokenize import generate_tokens

########################
###   TOKENS IN GENERATOR
########################

def tokens_in(s, strip_it=True):
    closure = {'processed': False}
    def get():
        if closure['processed']:
            return ''

        closure['processed'] = True
        if strip_it:
            return s.strip()
        else:
            return s
    return [(t, v) for t, v, _, _, _ in generate_tokens(get)][:-1]

########################
###   TOKENS
########################

class Tokens(object):
    def __init__(self, default_kls):
        self.default_kls = tokens_in(default_kls)

        self.equivalence = {
              'after_each' : 'tearDown'
            , 'before_each' : 'setUp'
            }

        self.before_each = [
              (NAME, 'def')
            , (NAME, self.equivalence['before_each'])
            , (OP, '(')
            , (NAME, 'self')
            , (OP, ')')
            ]

        self.after_each = [
              (NAME, 'def')
            , (NAME, self.equivalence['after_each'])
            , (OP, '(')
            , (NAME, 'self')
            , (OP, ')')
            ]

        self.test_skip = [
              (NAME, 'raise')
            , (NAME, 'nose.SkipTest')
            ]

    ########################
    ###   MAKERS
    ########################

    def make_single(self, name, args):
        lst = [ (NAME, 'def')
              , (NAME, name)
              , (OP, '(')
              ]

        if args:
            lst.extend(args[0])

            for arg in args[1:]:
                lst.append((OP, ','))
                lst.extend(arg)

        lst.extend(
            [ (OP, ')')
            , (OP, ':')
            ]
        )
        return lst

    def make_describe(self, kls, name):
        lst = [ (NAME, 'class')
                 , (NAME, name)
                 , (OP, '(')
                 ]
        if kls:
            lst.extend(tokens_in(kls))
        else:
            lst.extend(self.default_kls)

        lst.extend(
            [ (OP, ')')
            , (OP, ':')
            ]
        )

        return lst

    def make_super(self, indent, kls, method):
        if kls:
            kls = tokens_in(kls)
        else:
            kls = self.default_kls

        method_name = 'noy_sup_%s' % self.equivalence[method]
        result = [ (OP, ':')
                 , (NEWLINE, '\n')
                 , (INDENT, indent)
                 , (NAME, method_name)
                 , (OP, '(')
                 , (NAME, 'super')
                 , (OP, '(')
                 ]

        result.extend(kls)

        result.extend(
            [ (OP, ',')
            , (NAME, 'self')
            , (OP, ')')
            , (OP, ')')
            ]
        )

        return result

    def make_describe_attr(self, describe):
        return [ (NEWLINE, '\n')
               , (NAME, describe)
               , (OP, '.')
               , (NAME, 'is_noy_spec')
               , (OP, '=')
               , (NAME, 'True')
               ]

    def make_name_modifier(self, ismethod, cleaned, english):
        result = [ (NEWLINE, '\n') ]

        parts = cleaned.split('.')
        result.append((NAME, parts[0]))

        for part in parts[1:]:
            result.extend(
                [ (OP, '.')
                , (NAME, part)
                ]
            )

        if ismethod:
            result.extend(
                [ (OP, '.')
                , (NAME, "__func__")
                ]
            )

        result.extend(
            [ (OP, '.')
            , (NAME, "__testname__")
            , (OP, '=')
            , (STRING, english)
            ]
        )

        return result

    def wrap_setup(self, class_name, typ):
        """Described.typ = noy_wrap_typ(Described, Described.typ)"""
        equivalence = self.equivalence[typ]
        return [
              (NEWLINE, '\n')
            , (NAME, class_name)
            , (OP, '.')
            , (NAME, equivalence)
            , (OP, '=')
            , (NAME, "noy_wrap_%s" % equivalence)
            , (OP, "(")
            , (NAME, class_name)
            , (OP, ',')
            , (NAME, class_name)
            , (OP, '.')
            , (NAME, equivalence)
            , (OP, ")")
            ]

    def wrap_after_each(self, class_name):
        return self.wrap_setup(class_name, "after_each")

    def wrap_before_each(self, class_name):
        return self.wrap_setup(class_name, "before_each")




from tokenize import OP, NEWLINE
from tokens import tokens_in

def determine_imports(with_default_imports=False, with_should_dsl=False, extra_imports=None):
    default = []

    if extra_imports:
        if type(extra_imports) in (str, unicode):
            default.extend(tokens_in(extra_imports))
        else:
            for d in extra_imports:
                if d[0] == NEWLINE:
                    # I want to make sure the extra imports don't
                    # Take up extra lines in the code than the "# coding: spec"
                    default.append((OP, ';'))
                else:
                    default.append(d)

    if with_default_imports:
        if default and tuple(default[-1]) != (OP, ';'):
            default.append((OP, ';'))

        should_dsl = ""
        if with_should_dsl:
            should_dsl = "from should_dsl import *;"

        default.extend(
            tokens_in('import nose; from nose.tools import *; %s from noseOfYeti.tokeniser.support import *' % should_dsl)
        )

    return default




from should_dsl import should
import fudge

from noseOfYeti.tokeniser.config import Default, Config
from helpers import a_temp_file

# Silencing code checker about should_dsl matchers
be = None
equal_to = None

class Test_Default(object):

    def test_stringify_returns_value(self):
		str(Default("blah")) |should| equal_to("blah")
		unicode(Default("blah")) |should| equal_to("blah")

		str(Default(1)) |should| equal_to("1")
		unicode(Default(1)) |should| equal_to("1")

		class Special(object):
			def __unicode__(self):
				return 'things'
			def __str__(self):
				return 'stuff'
		str(Default(Special())) |should| equal_to("stuff")
		unicode(Default(Special())) |should| equal_to("things")

class Test_ConfigSetup(object):
	'''Test that the Config class knows how to setup itself'''
	def test_it_updates_values_with_options(self):
		# With no values to begin with
		config = Config()
		config._util.values |should| equal_to({})
		config.setup({'a':1, 'b':2})
		config._util.values |should| equal_to({'a':1, 'b':2})

	def test_it_uses_extractor_if_provided(self):
		'''If given an extractor, it applies extractor to the options before setting them'''
		template = fudge.Fake("template")
		def extractor(template, options):
			for key, val in options.items():
				yield key, val + 1

		config = Config(template)
		config.setup({'a':1, 'b':2}, extractor=extractor)
		config._util.values |should| equal_to({'a':2, 'b':3})

	def test_it_gets_values_from_config_file(self):
		'''It gets values from a config file'''
		contents = """
			{ "a" : 1
			, "b" : 2
			}
		"""
		with a_temp_file(contents) as filename:
			config = Config()
			config.setup({"config-file":filename})
			config._util.values |should| equal_to({"config_file":filename, u"a":1, u"b":2})

	def test_it_overwrites_config_file_with_options(self):
		'''It gets values from a config file'''
		contents = """
			{ "a" : 1
			, "b" : 2
			}
		"""
		with a_temp_file(contents) as filename:
			config = Config()
			config.setup({"config-file":filename, "a":4})
			config._util.values |should| equal_to({"config_file":filename, "a":4, u"b":2})

	@fudge.patch("noseOfYeti.tokeniser.config.ConfigUtil")
	def test_setup_uses_util(self, FakeConfigUtil):
		'''It uses self._util in setup'''
		options = fudge.Fake("options")
		template = fudge.Fake("template")
		extractor = fudge.Fake("extractor")
		util = (fudge.Fake("util").remember_order()
			.expects("use_options").with_args(options, extractor)
			.expects("use_config_file")
			)

		FakeConfigUtil.expects_call().with_args(template).returns(util)
		config = Config(template)
		config.setup(options, extractor=extractor)

class Test_ConfigAttributes(object):
	'''Make sure you can get values from config like they were attributes'''

	def test_config_gets_values_like_attributes(self):
		'''Make sure the options we set can be retrieved like values'''
		config = Config()
		config.setup({"a":1, "b":2, "c-d":3})

		config.a |should| be(1)
		config.b |should| be(2)
		config.c_d |should| be(3)

	@fudge.test
	def test_config_gets_attributes_from_util(self):
		'''__getattr__ on Config uses values from util'''
		key = 'my_awesome_key'
		val = fudge.Fake("val")

		util = fudge.Fake("util").expects("find_value").with_args(key).returns(val)
		config = Config()
		config._util = util

		getattr(config, key) |should| be(val)




from setuptools import setup

setup( name = 'noseOfYeti'
     , version = "1.4.7"
     , classifiers=[ 'Intended Audience :: Developers'
                   , 'Programming Language :: Python'
                   , 'Topic :: Software Development :: Documentation'
                   , 'Topic :: Software Development :: Testing'
                   ]

     , keywords='bdd rspec spec'
     , author = 'Stephen Moore'
     , author_email = 'delfick755@gmail.com'
     , license = 'GPL'
     , url = "https://github.com/delfick/nose-of-yeti"

     , description = 'Nose plugin providing BDD dsl for python'
     , long_description = """\
Plugin for nose, inspired by http://github.com/fmeyer/yeti, which uses a codec style
to provide an RSpec style BDD dsl for python tests
"""
     , install_requires = [ 'setuptools'
                          , 'should-dsl'
                          , 'nose'
                          , 'fudge'
                          ]

     , packages = [
          'noseOfYeti'
        , 'noseOfYeti.specs'
        , 'noseOfYeti.plugins'
        , 'noseOfYeti.tokeniser'
        , 'noseOfYeti.plugins.support'
        ]

     , entry_points = {
        'nose.plugins': [
            'noseOfYeti = noseOfYeti.plugins.nosetests:Plugin'
            ]
       }
)




from contextlib import contextmanager
from textwrap import dedent
import tempfile
import shutil
import os

@contextmanager
def a_temp_file(contents):
	tempfle = None
	try:
		tempfle = tempfile.NamedTemporaryFile(delete=False)
		with open(tempfle.name, 'w') as fle:
			fle.write(dedent(contents))

		yield tempfle.name
	finally:
		if tempfle:
			if os.path.exists(tempfle.name):
				os.remove(tempfle.name)

@contextmanager
def a_temp_dir():
	tempdir = None
	try:
		tempdir = tempfile.mkdtemp()
		yield tempdir
	finally:
		if tempdir:
			if os.path.exists(tempdir):
				shutil.rmtree(tempdir)




from should_dsl import should
import fudge

from noseOfYeti.tokeniser.config import Default, ConfigUtil, MissingConfigFile
from helpers import a_temp_file

# Silencing code checker about should_dsl matchers
be = None
throw = None
equal_to = None

def test_config_util_initialisation():
	'''Test ConfigUtil initialises properly'''
	template = fudge.Fake("template")
	config_util = ConfigUtil(template)

	config_util.values |should| equal_to({})
	config_util.template |should| be(template)

def test_config_util_normalising_keys():
	'''Test ConfigUtil knows how to normalise a key'''
	config_util = ConfigUtil()
	config_util.normalise_key("blah") |should| equal_to("blah")
	config_util.normalise_key("blah_stuff-things") |should| equal_to("blah_stuff_things")
	config_util.normalise_key("blah-stuff-things") |should| equal_to("blah_stuff_things")

class Test_ConfigUtil_FindingValue(object):
	'''Make sure we can find a value'''
	def test_it_raises_attributeError_when_key_doesnt_exist(self):
		config_util = ConfigUtil()
		config_util.values = {"a":1}

		config_util.find_value("a") |should| be(1)

		finder = lambda : config_util.find_value("b")
		finder |should| throw(AttributeError, "Config has no value for b")

	def test_it_returns_actual_value_if_finds_default(self):
		config_util = ConfigUtil()
		config_util.values = {"a":Default(21)}
		config_util.find_value("a") |should| be(21)

	def test_it_returns_value_as_is(self):
		val = fudge.Fake("val")
		config_util = ConfigUtil()
		config_util.values = {"c":val}
		config_util.find_value("c") |should| be(val)

class Test_ConfigUtil_UsingOptions(object):
	'''Make sure it knows how to use options'''
	def setup(self):
		self.config_util = ConfigUtil()
		self.config_util.values |should| equal_to({})

	def test_it_puts_options_in_values(self):
		self.config_util.use_options({"a":1, "b":2})
		self.config_util.values |should| equal_to({"a":1, "b":2})

	def test_it_overrides_all_values(self):
		c_val = Default(3)
		self.config_util.use_options({"a":1, "b":2, "c":c_val})
		self.config_util.values |should| equal_to({"a":1, "b":2, "c":c_val})

		self.config_util.use_options({"b":4, "c":5, "d":6})
		self.config_util.values |should| equal_to({"a":1, "b":4, "c":5, "d":6})

	def test_it_normalises_keys(self):
		self.config_util.use_options({"a-b":1, "c_d":2, "e-f_g-h":3})
		self.config_util.values |should| equal_to({"a_b":1, "c_d":2, "e_f_g_h":3})

	def test_it_uses_extractor_if_provided(self):
		template = fudge.Fake("template")
		def extractor(templ, values):
			templ |should| be(template)
			for key, val in values.items():
				yield "e-{}".format(key), val + 1

		self.config_util.template = template
		self.config_util.use_options({"a":1, "b":2, "c":3}, extractor=extractor)
		self.config_util.values |should| equal_to({"e_a":2, "e_b":3, "e_c":4})

	def test_it_works_with_list_of_tuples(self):
		c_val = Default(3)
		self.config_util.use_options([("a",1), ("b",2), ("c",c_val)])
		self.config_util.values |should| equal_to({"a":1, "b":2, "c":c_val})

	def test_it_doesnt_complain_if_extractor_returns_none(self):
		'''It's valid for the extractor to say there are no values'''
		template = fudge.Fake("template")
		def extractor(templ, values):
			templ |should| be(template)
			return None

		self.config_util.template = template
		self.config_util.use_options({"a":1, "b":2, "c":3}, extractor=extractor)
		self.config_util.values |should| equal_to({})

class Test_ConfigUtil_FindingConfigFile(object):
	'''Test that the ConfigUtil class knows corner cases of finding a config file'''
	def setUp(self):
		self.config_util = ConfigUtil()
		self.fullpath = fudge.Fake('fullpath')
		self.filename = fudge.Fake('filename')

	@fudge.patch('os.path.exists', 'os.path.abspath')
	def test_it_complains_if_config_file_doesnt_exist(self, fake_exists, fake_abspath):
		self.config_util.values['config_file'] = self.filename
		fake_abspath.expects_call().with_args(self.filename).returns(self.fullpath)
		fake_exists.expects_call().with_args(self.fullpath).returns(False)

		finder = lambda : self.config_util.find_config_file()
		finder |should| throw(MissingConfigFile, "Config file doesn't exist at fake:fullpath")

	@fudge.patch('os.path.exists', 'os.path.abspath')
	def test_it_defaults_to_noyjson(self, fake_exists, fake_abspath):
		fake_abspath.expects_call().with_args('noy.json').returns(self.fullpath)
		fake_exists.expects_call().with_args(self.fullpath).returns(True)
		self.config_util.find_config_file() |should| equal_to(self.fullpath)

	@fudge.patch('os.path.exists', 'os.path.abspath')
	def test_it_doesnt_care_if_default_doesnt_exist(self, fake_exists, fake_abspath):
		fake_abspath.expects_call().with_args('noy.json').returns(self.fullpath)
		fake_exists.expects_call().with_args(self.fullpath).returns(False)
		self.config_util.find_config_file() |should| equal_to(None)

	@fudge.patch('os.path.exists', 'os.path.abspath')
	def test_it_doesnt_care_if_provided_default_doesnt_exist(self, fake_exists, fake_abspath):
		self.config_util.values['config_file'] = Default(self.filename)
		fake_abspath.expects_call().with_args(self.filename).returns(self.fullpath)
		fake_exists.expects_call().with_args(self.fullpath).returns(False)
		self.config_util.find_config_file() |should| equal_to(None)

class Test_ConfigUtil_ApplyingConfigFile(object):
	'''Make sure it knows what values from the config file to apply'''
	def test_it_sets_options_from_an_json_file(self):
		contents = """
			{ "a" : "stuff"
			, "b" : "things"
			}
		"""
		with a_temp_file(contents) as filename:
			config_util = ConfigUtil()
			config_util.values |should| equal_to({})
			config_util.apply_config_file(filename)
			config_util.values |should| equal_to({"a":"stuff", "b":"things"})

	def test_it_doesnt_override_existing_non_default_values(self):
		contents = """
			{ "a" : "stuff"
			, "b" : "things"
			, "c" : "and"
			, "d" : "shells"
			, "e-f" : "blah"
			, "g-h" : "other"
			}
		"""
		with a_temp_file(contents) as filename:
			config_util = ConfigUtil()
			c_val = Default(21)
			config_util.use_options({'b':21, 'c':c_val, 'd':None, "g-h":"meh"})
			config_util.values |should| equal_to({'b':21, 'c':c_val, 'd':None, "g_h":"meh"})

			config_util.apply_config_file(filename)
			config_util.values |should| equal_to({"a":"stuff", "b":21, "c":"and", 'd':None, "e_f":"blah", "g_h":"meh"})

class Test_ConfigUtil_UsingConfigFile(object):
	'''Make sure it knows how to find and apply a config file'''
	@fudge.test
	def test_it_doesnt_care_if_no_config_file(self):
		config_util = ConfigUtil()
		fake_find_config_file = fudge.Fake("find_config_file").expects_call().returns(None)
		fake_apply_config_file = fudge.Fake("apply_config_file")

		with fudge.patched_context(config_util, 'find_config_file', fake_find_config_file):
			with fudge.patched_context(config_util, 'apply_config_file', fake_apply_config_file):
				config_util.use_config_file()

	@fudge.test
	def test_it_applies_config_file_if_one_is_found(self):
		config_util = ConfigUtil()
		config_file = fudge.Fake("config_file")
		fake_find_config_file = fudge.Fake("find_config_file").expects_call().returns(config_file)
		fake_apply_config_file = fudge.Fake("apply_config_file").expects_call().with_args(config_file)

		with fudge.patched_context(config_util, 'find_config_file', fake_find_config_file):
			with fudge.patched_context(config_util, 'apply_config_file', fake_apply_config_file):
				config_util.use_config_file()




from noseOfYeti.tokeniser.spec_codec import register_from_options
from noseOfYeti.tokeniser.config import Default
from support.spec_options import spec_options

import os

def enable(app):
    register_from_options(app.builder.config, spec_options, extractor=extract_options)

def normalise_options(template):
    env = os.environ
    for option, attributes in template.items():
        name = 'noy_{}'.format(option.replace('-', '_'))
        yield option, name, Default(attributes['default'](env))

def extract_options(template, config):
    for option, name, _ in normalise_options(template):
        yield option, getattr(config, name)

def setup(app):
    for option, name, default in normalise_options(spec_options):
        app.add_config_value(name, default, 'html')

    app.connect('builder-inited', enable)




from noseOfYeti.tokeniser.spec_codec import register_from_options
from noseOfYeti.tokeniser.config import Default
from support.spec_options import spec_options
from support.test_chooser import TestChooser

from nose.plugins import Plugin

def add_to_argparse(parser, env, template):
    parser_options = ['default', 'action', 'dest', 'help']
    for option, attributes in template.items():
        opts = dict((k, v) for k, v in attributes.items() if k in parser_options)
        opts['default'] = Default(opts['default'](env))
        parser.add_option('--noy-%s' % option, **opts)

def extract_options(template, options):
    for option, val in template.items():
        name = option.replace('-', '_')
        yield option, getattr(options, name)

class Plugin(Plugin):
    name = "noseOfYeti"

    def __init__(self, *args, **kwargs):
        self.test_chooser = TestChooser()
        super(Plugin, self).__init__(*args, **kwargs)

    def options(self, parser, env={}):
        super(Plugin, self).options(parser, env)
        add_to_argparse(parser, env, spec_options)

        parser.add_option('--with-noy'
            , default = False
            , action  = 'store_true'
            , dest    = 'enabled'
            , help    = 'Enable nose of yeti'
            )

        default_ignore_kls = []
        if 'NOSE_NOY_IGNORE_KLS' in env:
            default_ignore_kls.append(env['NOSE_NOY_IGNORE_KLS'].split(','))

        parser.add_option('--noy-ignore-kls'
            , default = default_ignore_kls
            , action  = 'append'
            , dest    = 'ignore_kls'
            , help    = 'Set class name to ignore in wantMethod'
            )

    def wantModule(self, mod):
        self.test_chooser.new_module()

    def wantMethod(self, method):
        return self.test_chooser.consider(method, self.ignore_kls)

    def configure(self, options, conf):
        super(Plugin, self).configure(options, conf)
        self.ignore_kls = options.ignore_kls

        if options.enabled:
            self.done = {}
            self.enabled = True
            register_from_options(options, spec_options, extractor=extract_options)

